


#include <xc.h>
#include "dimmer.h"      
#include "boutonsq.h"

    
#define DIMM_TMR_PERIOD_US      250
#define DIMM_PULSE_MAINTIENT    3       //jusqu' x on maintient le pulse pour faire conduire le triac
#define DIMM_TAB_SIZE           DIMM_STEP_PER_DEMI_ER

#define DIM_CHAN0_MASK          0x08
#define DIM_CHAN1_MASK          0x04
#define DIM_CHAN2_MASK          0x02
#define DIM_CHAN3_MASK          0x01


uint8 u8_dimm_table[DIMM_TAB_SIZE];
uint8 u8_dimm_PtrNext;
uint8 u8_dimm_FlagZeroCrossed;




    DIMMER  dimmer_chan0;
    DIMMER  dimmer_chan1;
    DIMMER  dimmer_chan2;
    DIMMER  dimmer_chan3;





#define SMART_PULSE     // only one pulse
#define FULL_PULSE      // pulse for all ON time






#define VAL_OFF     (DIMM_STEP_PER_DEMI_ER+1)
#define VAL_MIN     (DIMM_STEP_PER_DEMI_ER-12)
#define VAL_MAX     (1)


void Val_diminue ( DIMMER *li_PtrDimChan){
    if ( li_PtrDimChan->val < VAL_MIN){
        li_PtrDimChan->val++;
    }
}
void Val_Augmente ( DIMMER *li_PtrDimChan){
    if ( li_PtrDimChan->val >= VAL_MAX){
        li_PtrDimChan->val--;
    }
}
void Val_Eteint ( DIMMER *li_PtrDimChan){
    li_PtrDimChan->val = VAL_OFF;
}
void Val_Allume ( DIMMER *li_PtrDimChan){
    li_PtrDimChan->val = VAL_MAX;
}






//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void dimmer_init (){
    dimmer_setdimm ( DIMM_CHAN_0, DIMM_STEP_PER_DEMI_ER);
    dimmer_setdimm ( DIMM_CHAN_1, DIMM_STEP_PER_DEMI_ER);
    dimmer_setdimm ( DIMM_CHAN_2, DIMM_STEP_PER_DEMI_ER);
    dimmer_setdimm ( DIMM_CHAN_3, DIMM_STEP_PER_DEMI_ER);
    u8_dimm_PtrNext = 0;
    u8_dimm_FlagZeroCrossed = 0;
    
    
    HWI_timer_cfg ( HWI_TIMER_CHAN_1 , DIMM_TMR_PERIOD_US);
	HWI_timer_reload ( HWI_TIMER_CHAN_1 );
	HWI_timer_enable ( HWI_TIMER_CHAN_1 );
    PIR1bits.TMR1IF = 0;
    PIE1bits.TMR1IE = 1;
    
    dimmer_chan0.etat = DIMMER_ETAT_OFF;
    dimmer_chan1.etat = DIMMER_ETAT_OFF;
    dimmer_chan2.etat = DIMMER_ETAT_OFF;
    dimmer_chan3.etat = DIMMER_ETAT_OFF;
    

    Val_Eteint ( &dimmer_chan0 );
    Val_Eteint ( &dimmer_chan1 );
    Val_Eteint ( &dimmer_chan2 );
    Val_Eteint ( &dimmer_chan3 );
    
    TRISD = 0x30;
    
}

//-----------------------------------------------------------------------------
//      a call lors du passage par zero
//-----------------------------------------------------------------------------
void dimmer_SignalZeroCross (){
    HWI_timer_reload ( HWI_TIMER_CHAN_1 );
    u8_dimm_FlagZeroCrossed = 1;
    u8_dimm_PtrNext = 0;
    PORTD &= ~( DIM_CHAN0_MASK | DIM_CHAN1_MASK | DIM_CHAN2_MASK | DIM_CHAN3_MASK );       //stop les pulses
}

//-----------------------------------------------------------------------------
//      definir la luminosit
//-----------------------------------------------------------------------------
void dimmer_setdimm ( uint8 li_u8_chan, uint8 li_u8_val){
    uint8   l_u8_mask;
    uint8   l_u8_cpt;
    
    switch ( li_u8_chan ){
        default:
        case DIMM_CHAN_0:
            l_u8_mask = DIM_CHAN0_MASK;
            break;
        case DIMM_CHAN_1:
            l_u8_mask = DIM_CHAN1_MASK;
            break;
        case DIMM_CHAN_2:
            l_u8_mask = DIM_CHAN2_MASK;
            break;
        case DIMM_CHAN_3:
            l_u8_mask = DIM_CHAN3_MASK;
            break;
    };    
    
    if ( li_u8_val >= DIMM_STEP_PER_DEMI_ER){
        
        for ( l_u8_cpt=0 ; l_u8_cpt<DIMM_STEP_PER_DEMI_ER ; l_u8_cpt++ ){
            u8_dimm_table[l_u8_cpt] &= ~l_u8_mask;
        }
        return;
    }
    
#ifdef FULL_PULSE
    for ( l_u8_cpt=0 ; l_u8_cpt<DIMM_STEP_PER_DEMI_ER ; l_u8_cpt++ ){
        if ( l_u8_cpt < li_u8_val ){
            u8_dimm_table[l_u8_cpt] &= ~l_u8_mask;
        }else{
            u8_dimm_table[l_u8_cpt] |= l_u8_mask;
        }
    }
#endif
 
    

#ifdef SMART_PULSE
    //-- mettre d'abord la pulse, en conservant l'ancienne pulse, puis supprimer les anciennes
    // cela permettra de ne pas se retrouver sans pulses et donc de ne pas bloques les IT pendant 
    // la mise  jour
    u8_dimm_table[li_u8_val] |= l_u8_mask;
    
    //-- enlever les anciennes pulses cad parcourir et mettre  0
    for ( l_u8_cpt=0 ; l_u8_cpt<DIMM_STEP_PER_DEMI_ER ; l_u8_cpt++ ){
        if ( l_u8_cpt != li_u8_val ){
            u8_dimm_table[l_u8_cpt] &= ~l_u8_mask;
        }
    }
    
    //--- pulses de maintien
    if ( li_u8_val <= DIMM_PULSE_MAINTIENT){
        for ( l_u8_cpt=0 ; l_u8_cpt<=li_u8_val ; l_u8_cpt++ ){
            u8_dimm_table[l_u8_cpt] |= l_u8_mask;
        }
    }
    /*
    if ( li_u8_val >= (DIMM_STEP_PER_DEMI_ER-DIMM_PULSE_MAINTIENT) ){
        for ( l_u8_cpt=li_u8_val ; l_u8_cpt<DIMM_STEP_PER_DEMI_ER ; l_u8_cpt++ ){
            u8_dimm_table[l_u8_cpt] |= l_u8_mask;
        }
    }
    */
    
#endif
    
}


//-----------------------------------------------------------------------------
//      a call dans l'IT toutes les 500 s
//-----------------------------------------------------------------------------
void dimmer_it_handler (){
    static uint8 l_s=0;
    uint8   l_u8_sav;
    
    HWI_timer_reload ( HWI_TIMER_CHAN_1 );

    
    //PORTDbits.RD0 = 1;
    
    if ( 1 == u8_dimm_FlagZeroCrossed ){
        l_u8_sav = PORTD & ~( DIM_CHAN0_MASK | DIM_CHAN1_MASK | DIM_CHAN2_MASK | DIM_CHAN3_MASK );
        PORTD = u8_dimm_table[u8_dimm_PtrNext++] | l_u8_sav;
        if ( u8_dimm_PtrNext >= DIMM_STEP_PER_DEMI_ER ){
            u8_dimm_PtrNext = 0;
            u8_dimm_FlagZeroCrossed = 0;
            
        }
        
    }else{
        
    }
    //PORTDbits.RD0 = 0;
}


//-----------------------------------------------------------------------------
//      
//-----------------------------------------------------------------------------
void dimmer_gest_btn_chan ( DIMMER *li_PtrDimChan, uint8 li_u8_order ){
    switch( li_PtrDimChan->etat ){
        default:
        case DIMMER_ETAT_OFF:
            if ( CMD_APPUI_SHORT == li_u8_order ){
                li_PtrDimChan->etat = DIMMER_ETAT_ON;
                li_PtrDimChan->sens = 0;    //monte
                Val_Allume (li_PtrDimChan);
            }
            if ( CMD_APPUI_LONG == li_u8_order ){
                li_PtrDimChan->etat = DIMMER_ETAT_ON;
                li_PtrDimChan->sens = 1;    //monte
            }
            break;
        case DIMMER_ETAT_ON:
            if ( CMD_APPUI_SHORT == li_u8_order  ){
                li_PtrDimChan->etat = DIMMER_ETAT_OFF;
                li_PtrDimChan->sens = 1;    //monte
                Val_Eteint (li_PtrDimChan);
            }
            if ( CMD_APPUI_LONG == li_u8_order ){
                if (1 == li_PtrDimChan->sens){
                    Val_Augmente (li_PtrDimChan);
                }else{
                    Val_diminue (li_PtrDimChan);
                }
            }
            if ( CMD_APPUI_RELACHE == li_u8_order ){
                if (1 == li_PtrDimChan->sens ){
                    li_PtrDimChan->sens = 0;
                }else{
                    li_PtrDimChan->sens = 1;
                }
            }
            break;
    }
}


//-----------------------------------------------------------------------------
//      
//-----------------------------------------------------------------------------
void dimmer_gest_btn ( void ){
    dimmer_gest_btn_chan ( &dimmer_chan0, btn_0.ordre );
    dimmer_gest_btn_chan ( &dimmer_chan1, btn_1.ordre );
    dimmer_gest_btn_chan ( &dimmer_chan2, btn_2.ordre );
    dimmer_gest_btn_chan ( &dimmer_chan3, btn_3.ordre );
    
    btn_0.ordre = CMD_APPUI_NONE;
    btn_1.ordre = CMD_APPUI_NONE;
    btn_2.ordre = CMD_APPUI_NONE;
    btn_3.ordre = CMD_APPUI_NONE;
    
    dimmer_setdimm ( DIMM_CHAN_0, dimmer_chan0.val);
    dimmer_setdimm ( DIMM_CHAN_1, dimmer_chan1.val);
    dimmer_setdimm ( DIMM_CHAN_2, dimmer_chan2.val);
    dimmer_setdimm ( DIMM_CHAN_3, dimmer_chan3.val);
}


