//   ==============================================================
//   | HISTORIQUE:                                                |
//   |                                                            |
//   |     Date    - Description                                  |
//   |                                                            |
//   |  27/11/2013 - Reprise existant, mutation vers XC8          |
//   |                                                            |
//   ==============================================================




#include "hwi.h"
#include "hwi_eeprom.h"


//==============================================================
// @brief attends que l'eeprom soit prte avant de quitter
//
//==============================================================
void HWI_eeprom_busy ( void )
{
	while(EECON1bits.WR);
}


//==============================================================
//   ecriture de valeur type : LSB - MSB
//==============================================================
void HWI_eepprom_write_u16  ( uint16 badd, uint16 li_data ){
    uint8   l_u8;	//temporaire
    
    l_u8 = (uint8) ( li_data & 0x00FF );
    HWI_eeprom_busy ();
    HWI_write_byte_eeprom ( badd, l_u8 );
    l_u8 = (uint8) ( li_data >> 8 );
    HWI_eeprom_busy ();
    HWI_write_byte_eeprom ( badd+1, l_u8 );
}


//==============================================================
// @brief crit dans l'eeprom
// @param badd (entre) : adresse d'criture
// @param li_u8data (entre) : octet  crire
//
//==============================================================
void HWI_write_byte_eeprom( uint16 badd,uint8 li_u8data )
{

	EEADR = (badd & 0x0ff);
	EEDATA = li_u8data;
  	EECON1bits.EEPGD = 0;
	EECON1bits.CFGS = 0;
	EECON1bits.WREN = 1;
	INTCONbits.GIE = 0;
	EECON2 = 0x55;
	EECON2 = 0xAA;
	EECON1bits.WR = 1;
	INTCONbits.GIE = 1;
	EECON1bits.WREN = 0;
}

//==============================================================
// @brief lit dans l'eeprom
// @param badd (entre) : adresse d'criture
//
//==============================================================
uint8 HWI_read_byte_eeprom( uint16 badd )
{

	EEADR = (badd & 0x0ff);			// 1024 byte EEP
	EECON1bits.CFGS = 0;
	EECON1bits.EEPGD = 0;
	EECON1bits.RD = 1;
	return ( EEDATA );              // return with read byte 
}

//==============================================================
// @brief crit une chaine de char dans l'eeprom
// @param li_ptr (entre) : pointeur vers la chaine  crire
// @param li_size (entre) : taille de la chaine
//
//==============================================================
void HWI_write_string_eeprom( uint16 badd,uint8 *li_ptr, uint8 li_size ){
	uint8	l_cpt;		//compteur d'octets
	for (l_cpt=0 ; l_cpt<li_size ; l_cpt++)
	{
		HWI_eeprom_busy ();
		HWI_write_byte_eeprom( badd + (uint16) l_cpt, *(li_ptr+l_cpt) );
	}	
}	

//==============================================================
// @brief lit une chaine de char dans l'eeprom
// @param li_ptr (entre) : pointeur vers la chaine  crire
// @param li_size (entre) : taille de la chaine
//
//
//==============================================================
void HWI_read_string_eeprom( uint16 badd,uint8 *li_ptr, uint8 li_size ){
	uint8	l_cpt;		//compteur d'octets
	for (l_cpt=0 ; l_cpt<li_size ; l_cpt++)
	{
		HWI_eeprom_busy ();
		*(li_ptr+l_cpt) = HWI_read_byte_eeprom ( badd + (uint16) l_cpt);
	}	
}	
