//   ==============================================================
//   | HISTORIQUE:                                                |
//   |                                                            |
//   |     Date    - Description                                  |
//   |                                                            |
//   |  27/11/2013 - Reprise existant, mutation vers XC8          |
//   |                                                            |
//   ==============================================================


//==============================================================
//     FICHIERS A EN-TETE
//==============================================================
#include "hwi_timers.h"
#include "stdio.h"


#undef HWI_TMR_COMPENSE_IT_TIME

// tables contenant les valeurs des prescalers possibles pour les timers
// attention la valeur de fin avec double zero, doit toujours tre prsente
// configuration pour le timer 0 = chan 1
static HWI_Timer_prescaler_type	hwi_timer_0_pre[9] = {	{2, 0},	{4, 1},	{8, 2},	{16, 3}, {32, 4}, {64, 5}, {128, 6}, {256, 7}, {0, 0} };
// configuration pour le timer 1 = chan 2
static HWI_Timer_prescaler_type	hwi_timer_1_pre[5] = {	{1, 0},	{2, 0x10},	{4, 0x20},	{8, 0x30}, {0, 0} };
// configuration pour le timer 2 = chan 3
static HWI_Timer_prescaler_type	hwi_timer_2_pre[4] = {	{1, 0},	{4, 0x01},	{16, 0x02}, {0, 0} };
// configuration pour le timer 3 = chan 4
static HWI_Timer_prescaler_type	hwi_timer_3_pre[5] = {	{1, 0},	{2, 0x10},	{4, 0x20},	{8, 0x30}, {0, 0} };

// configuration pour le timer 0, (CHAN 0)
// 16 bits, prescaler jusque 1/256
// periode max : 1667 ms (quartz 10MHz + PLL)
static  	HWI_Timer_cfg_type	HWI_TIMER_CHAN_0_cfg = {
	HWI_TIMER_16_BITS,
	(HWI_Timer_prescaler_type*) hwi_timer_0_pre,
	(uint8*) &T0CON,
	0x07,
	(uint8*) &TMR0L,
	(uint8*) &TMR0H,
	0x80				// timer on en 16 bits
};

// configuration pour le timer 1, (CHAN 1)
// 16 bits, prescaler jusque 1/8
// periode max : 52 ms (quartz 10MHz + PLL)
static  	HWI_Timer_cfg_type	HWI_TIMER_CHAN_1_cfg = {
	HWI_TIMER_16_BITS,
	(HWI_Timer_prescaler_type*) hwi_timer_1_pre,
	(uint8*) &T1CON,
	0x30,
	(uint8*) &TMR1L,
	(uint8*) &TMR1H,
    0x01				// timer on en 16 bits
};

// configuration pour le timer 2 ( CHAN 2)
// 8 bits prescaler de 1  1/16
// periode max : 409us (quartz 10MHz + PLL)
static  	HWI_Timer_cfg_type	HWI_TIMER_CHAN_2_cfg = {
	HWI_TIMER_8_BITS,
	(HWI_Timer_prescaler_type*) hwi_timer_2_pre,
	(uint8*) &T2CON,
	0x03,
	(uint8*) &TMR2,
	(uint8*) &TMR2,
	0x04
};

// configuration pour le timer 3 ( CHAN 3)
// 16 bits, prescaler jusque 1/8
// periode max : 52 ms (quartz 10MHz + PLL)
static 	HWI_Timer_cfg_type	HWI_TIMER_CHAN_3_cfg = {
	HWI_TIMER_16_BITS,
	(HWI_Timer_prescaler_type*) hwi_timer_3_pre,
	(uint8*) &T3CON,
	0x30,
	(uint8*) &TMR3L,
	(uint8*) &TMR3H,
	0x81
};
//table de recapitulation des configurations.
static   HWI_Timer_cfg_type	*hwi_Timer_cfg_lut[] ={
	&HWI_TIMER_CHAN_0_cfg,
	&HWI_TIMER_CHAN_1_cfg,
	&HWI_TIMER_CHAN_2_cfg,
	&HWI_TIMER_CHAN_3_cfg
};

// stockage des paramtres de configuration pour le reload, juste une dclaration  faire par timer
static HWI_Timer_ram_param		hwi_timer_0_ram_param;
static HWI_Timer_ram_param		hwi_timer_1_ram_param;
static HWI_Timer_ram_param		hwi_timer_2_ram_param;
static HWI_Timer_ram_param		hwi_timer_3_ram_param;

static HWI_Timer_ram_param	*hwi_Timer_param_lut[] ={
	(  HWI_Timer_ram_param*) &hwi_timer_0_ram_param,
	(  HWI_Timer_ram_param*) &hwi_timer_1_ram_param,
	(  HWI_Timer_ram_param*) &hwi_timer_2_ram_param,
	(  HWI_Timer_ram_param*) &hwi_timer_3_ram_param
};	
			


//==============================================================
// @brief configuration du timer
//  cette fonction ne lance pas le timer, il faut appeler "enable_timer"
//  se rfrer au datasheet pour voir les erreurs suivant la periode demande
//==============================================================
void HWI_timer_cfg ( HWI_Timer_channel_type	li_channel, uint32 li_periode_us){
	uint8	l_cpt;                                      // compteur
	uint8	l_fin;                                      // fin de recherche
	
	uint32	l_verif;                // vrification 
	uint32	l_max_count;            // max count du timer 
	uint32  l_periode_ns;           // periode en ns de la base de temps
	uint16	l_tmr_prescaler_val;    // valeur du prescaler
	uint32	l_base_tmr;             // base timer cad avec prescaler
	uint32	l_temp;                 // temporaire
	
	uint16	l_valeur_tmr;           // valeur du timer
	uint16	l_uint16;               // temporaire
	uint16	l_best_error;           // meilleur erreur
	uint16	l_best_tmr_val;         // meilleure valeur timer
	uint8	l_best_tmr_pres;        // meilleure valeur prescaler

	HWI_Timer_cfg_type			*l_ptr_tmr_cfg;         // pointeur vers str
	HWI_Timer_prescaler_type	*l_ptr_tmr_prescale;    // pointeur vers str
	HWI_Timer_ram_param			*l_ptr_param_ram;       // pointeur vers str
	
	// si on a un mauvais canal
	if ( li_channel >= HWI_TIMER_CFG_END)
	{
		return ;	
	}
	
	// pointer vers la configuration du timer 
	l_ptr_tmr_cfg = ( HWI_Timer_cfg_type*) hwi_Timer_cfg_lut[li_channel];
	l_ptr_tmr_prescale = ( HWI_Timer_prescaler_type*) l_ptr_tmr_cfg->prescalers;
	
	// suivant la taille du timer
	if ( HWI_TIMER_16_BITS == l_ptr_tmr_cfg->tmr_type)
	{
		l_max_count = HWI_TIMER_MAX_COUNT_16_BITS;
	}
	else
	{
		l_max_count = HWI_TIMER_MAX_COUNT_8_BITS;	
	}	
	
	//compenser le temps de traitement des IT
#ifdef HWI_TMR_COMPENSE_IT_TIME
	if ( li_periode_us > ( 2 * HWI_INTERRUPT_RESPONSE_TIME_US)){
		li_periode_us -= HWI_INTERRUPT_RESPONSE_TIME_US;	
	}
#endif
	
	//parcourir les diviseurs
	l_fin = 0;
	l_best_error = 0xFFFF;
	l_best_tmr_val = 0x0FFF;
	l_periode_ns = li_periode_us * (uint32) 1000;
	
	// parcourir les prescalers et calculer les priodes possibles
	for (l_cpt=0 ; (l_cpt<HWI_TIMERS_MAX_PRESCALE_NB) && (l_ptr_tmr_prescale[l_cpt].diviseur != 0) && (0==l_fin) ; l_cpt++)
	{
		//base de temps en ns du timer
		l_tmr_prescaler_val = l_ptr_tmr_prescale[l_cpt].diviseur;
		l_base_tmr = OSC_TEMPS_CYCLE_NS * (uint32) l_tmr_prescaler_val;		
		

		//test si les temps max et min que l'on peut 
		//faire sont compris dans le temps que l'on veut
		if (  
			((l_base_tmr * HWI_TIMER_MARGE_COUNT) <l_periode_ns)&&						//temps minimum 
			((l_base_tmr * (l_max_count-HWI_TIMER_MARGE_COUNT))>l_periode_ns   )		//temps max
		){
			
			// chercher maintenant le diviseur qui convient
			l_valeur_tmr = l_periode_ns / l_base_tmr;
			
			l_verif = l_valeur_tmr * l_base_tmr; 	// calculer la periode en ns que a donne, pour calculer l'erreur						
			
			l_valeur_tmr = l_max_count - l_valeur_tmr;
			
			//calculer l'erreur, du moins sa valeur absolue
			if ( l_periode_ns > l_verif){
				l_temp = l_periode_ns - l_verif;
			}else{
				l_temp = l_verif - l_periode_ns;
			}	
			l_uint16 = (uint16) l_temp;
		
			// si on a trouv sans erreurs, stopper la recherche, sinon 
			// enregistrer les paramtres et tenter de trouver mieux
			if ( 0 == l_uint16){
				l_fin = 1;
				l_best_error = 0;
				l_best_tmr_val = l_valeur_tmr;
				l_best_tmr_pres = l_cpt;
			}else{
				if ( l_uint16 < l_best_error){
					l_best_error = l_uint16;
					l_best_tmr_val = l_valeur_tmr;
					l_best_tmr_pres = l_cpt;
				}	
			}

		}else{

		}
	}

	//printf ("\n\r choisie: div=%d val=%u  err=%u", l_best_tmr_pres, l_best_tmr_val, l_best_error);
	// stocker la configuration calcule pour le timer
	//printf ("\n\r choisie: div=%d val=%u  err=%u", l_best_tmr_pres, l_best_tmr_val, l_best_error);
	l_ptr_param_ram = ( HWI_Timer_ram_param *) hwi_Timer_param_lut [li_channel];
	l_ptr_param_ram->param_temps = (uint16) l_best_tmr_val;
	l_ptr_param_ram->param_prescaler_val = (uint8) l_ptr_tmr_prescale[l_best_tmr_pres].reg_value;
	
	return ;
}


//==============================================================
// reconfiguration du timer
//==============================================================
void HWI_timer_reload (HWI_Timer_channel_type	li_channel){
	HWI_Timer_cfg_type			*l_ptr_tmr_cfg;
	HWI_Timer_ram_param			*l_ptr_param_ram;

	if ( li_channel >= HWI_TIMER_CFG_END)
	{
		return ;	
	}
	
	l_ptr_tmr_cfg = ( HWI_Timer_cfg_type*) hwi_Timer_cfg_lut[li_channel];
	l_ptr_param_ram = ( HWI_Timer_ram_param* )hwi_Timer_param_lut[li_channel];

	if ( HWI_TIMER_16_BITS == l_ptr_tmr_cfg->tmr_type)
	{
		*l_ptr_tmr_cfg->ptr_reg_val_H = (uint8) (l_ptr_param_ram->param_temps / (uint16) 0x0100);
	}
	*l_ptr_tmr_cfg->ptr_reg_val_L = (uint8) (l_ptr_param_ram->param_temps & (uint16) 0x00FF);
	return ;
}	


//==============================================================
// activation du timer
//==============================================================
void HWI_timer_enable (HWI_Timer_channel_type	li_channel){
	HWI_Timer_cfg_type			*l_ptr_tmr_cfg;
	HWI_Timer_ram_param			*l_ptr_param_ram;
	uint8	*l_ptr_TxCON;
	
	// si on  a un canal erron ?
	if ( li_channel >= HWI_TIMER_CFG_END)
	{
		return ;	
	}
	
	// pointeur vers la table des paramtres
	l_ptr_tmr_cfg = ( HWI_Timer_cfg_type*) hwi_Timer_cfg_lut[li_channel];

	// configurer le registre de priode
	// si on est en 16 bits il faut mettre 2 char
	if ( HWI_TIMER_16_BITS == l_ptr_tmr_cfg->tmr_type)
	{
		*l_ptr_tmr_cfg->ptr_reg_val_H = (uint8) (hwi_Timer_param_lut[li_channel]->param_temps / (uint16) 0x0100);
	}
	*l_ptr_tmr_cfg->ptr_reg_val_L = (uint8) (hwi_Timer_param_lut[li_channel]->param_temps & (uint16) 0x00FF);
	
	// activation du timer et configuration avec le prescaler
	l_ptr_param_ram = ( HWI_Timer_ram_param*)  hwi_Timer_param_lut[li_channel];
	l_ptr_TxCON = l_ptr_tmr_cfg->ptr_reg_TCON;
	*l_ptr_TxCON = 0;
	*l_ptr_TxCON &= ~l_ptr_tmr_cfg->prescaler_mask;
	*l_ptr_TxCON |= l_ptr_param_ram->param_prescaler_val;
	*l_ptr_TxCON |= l_ptr_tmr_cfg->on_val;
	
	return ;
}	


//==============================================================
//  desactivation du timer
//
//==============================================================
void HWI_timer_disable (HWI_Timer_channel_type	li_channel){
		HWI_Timer_cfg_type			*l_ptr_tmr_cfg;
	uint8	*l_ptr_TxCON;
	
	// si on  a un canal erron ?
	if ( li_channel >= HWI_TIMER_CFG_END)
	{
		return ;	
	}
	
	// pointeur vers la configuration du timer
	l_ptr_tmr_cfg = ( HWI_Timer_cfg_type*) hwi_Timer_cfg_lut[li_channel];
	l_ptr_TxCON = l_ptr_tmr_cfg->ptr_reg_TCON;

	// dsactivation
	*l_ptr_TxCON &= ~l_ptr_tmr_cfg->on_val;
	return ;
}
